from setuptools import setup
import setuptools

require_packages = ['absl-py==0.15.0',
'astunparse==1.6.3',
'autopep8==1.6.0',
'backports.functools-lru-cache==1.6.4',
'bleach==4.1.0',
'build==0.7.0',
'cachetools==4.2.4',
'certifi==2021.10.8',
'charset-normalizer==2.0.7',
'clang==5.0',
'click==7.1.2',
'colorama==0.4.4',
'docutils==0.17.1',
'flatbuffers==1.12',
'gast==0.4.0',
'google-auth==2.3.0',
'google-auth-oauthlib==0.4.6',
'google-pasta==0.2.0',
'grpcio==1.41.0',
'h5py==3.1.0',
'idna==3.3',
'importlib-metadata==4.8.1',
'indic-transliteration==2.3.2',
'joblib==1.1.0',
'keras==2.6.0',
'Keras-Preprocessing==1.1.2',
'keyring==23.2.1',
'Markdown==3.3.4',
'nltk==3.4.5',
'numpy==1.19.5',
'oauthlib==3.1.1',
'opt-einsum==3.3.0',
'packaging==21.0',
'pandas==1.3.4',
'pbr==2.1.0',
'pep517==0.12.0',
'pkginfo==1.7.1',
'polyglot-tokenizer==2.0.2',
'protobuf==3.19.0',
'pyasn1==0.4.8',
'pyasn1-modules==0.2.8',
'pycodestyle==2.8.0',
'Pygments==2.10.0',
'pyiwn==0.0.5',
'pyparsing==3.0.1',
'python-crfsuite==0.9.7',
'python-dateutil==2.8.2',
'pytz==2021.3',
'pywin32-ctypes==0.2.0',
'readme-renderer==30.0',
'regex==2020.10.11',
'requests==2.26.0',
'requests-oauthlib==1.3.0',
'requests-toolbelt==0.9.1',
'rfc3986==1.5.0',
'rsa==4.7.2',
'scikit-learn==1.0',
'scipy==1.7.1',
'selenium==3.141.0',
'seqeval==1.2.2',
'six==1.15.0',
'spello==1.2.0',
'tensorboard==2.7.0',
'tensorboard-data-server==0.6.1',
'tensorboard-plugin-wit==1.8.0',
'tensorflow==2.6.0',
'tensorflow-estimator==2.6.0',
'termcolor==1.1.0',
'threadpoolctl==3.0.0',
'toml==0.10.2',
'tomli==1.2.1',
'tqdm==4.62.3',
'twine==3.4.2',
'typer==0.3.2',
'typing-extensions==3.7.4.3',
'urllib3==1.26.7',
'webencodings==0.5.1',
'Werkzeug==2.0.2',
'wrapt==1.12.1',
'zipp==3.6.0']

setup(
    name = 'hindiwsd', 
    version = '1.0.9',
    author = 'Praatibh Surana, Mirza Yusuf',
    author_email = 'praatibhsurana@gmail.com', 
    description = 'A pipeline for transliteration of hinglish code mixed data to hindi along with spell correction and word sense disambiguation of hindi words.', 
    url = 'https://github.com/praatibhsurana/Hinglish_Hindi_WSD',
    #packages=['callbacks', 'convert_encoding', 'convert_ssf_conll', 'crf_chunk_features', 'crf_pos_features', 'CRF', 'data_reader', 'evaluate', 'generate_features', 'hindi_wsd', 'hipos', 'layers', 'lesks', 'models', 'pipeline', 'preprocessing', 'spacy_data_reader', 'spacy_tagger_test', 'spacy_tagger', 'tagger', 'trainer', 'utils', 'wrapper', 'writer', 'wsd'],
    packages = setuptools.find_packages(where="src"),
    install_requires=require_packages,
    package_data={'hindiwsd':['*.model', '*.pkl', '*.txt', '*.tsv']},
    include_package_data=True,
    package_dir={"":"src"},
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    keywords='nlp, machine learning, spell correction, IndoWordNet, Hindi, Hinglish code mixed to Hindi transliteration, word sense disambiguation',
)

# print(setuptools.find_packages(include=['hindi_wsd.']))